# Copyright 2022 Mattia Giambirtone & All Contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

## Reusable objects to simplify code generation using templates

type
    GeneratorKind* = enum
        ## A code generator enumeration
        Literal, Call
    CodeGenerator* = object
        ## A generic code generator
        case kind*: GeneratorKind
            of Literal:
                lit: string
            else:
                discard


proc generate*(self: CodeGenerator): string =
    ## Generates the source code for the given
    ## code generator object and returns it
    case self.kind:
        of Literal:
            return self.lit
        else:
            discard
    